/******************************************************************************
*  TAPI Example: Enumerates all TAPI devices and displays the device names in *
*                a listbox. Each device's capability information is saved     *
*                with the listbox item for viewing.                           *
*                                                                             *
*                            1995-1998 by Mark Gamber                         *
******************************************************************************/

#define WIN32_LEAN_AND_MEAN

#include "windows.h"
#include "windowsx.h"
#include "tapi.h"

// === Function Prototypes ====================================================

BOOL WINAPI MainDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL WINAPI ShowInfoProc( HWND, UINT, WPARAM, LPARAM );
BOOL GetTAPIDevices( HWND );
BOOL DeleteAllDevices( HWND );
VOID CALLBACK TapiCallbackProc( HANDLE, DWORD, DWORD, DWORD, DWORD, DWORD );

// === Global Variables =======================================================

HINSTANCE hInst;                                //  Application instance handle

// === Application Entry Point ================================================

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmd,
                      int nShow )
{
   hInst = hInstance;                    //  Save instance and call dialog box
   DialogBox( hInst, MAKEINTRESOURCE( 10000 ), NULL, MainDlgProc );
   return( FALSE );                          //  End app when dialog is closed
}

// === Main Dialog Proc =======================================================

BOOL WINAPI MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
   switch( msg )
   {
      case WM_INITDIALOG:
         return( TRUE );

      case WM_COMMAND:
         if( wParam == IDCANCEL )       //  Close dialog when Cancel is pressed
         {                              //  Free any alloced memory for devices
            DeleteAllDevices( GetDlgItem( hDlg, 1000 ) );
            EndDialog( hDlg, TRUE );                     //  And end the dialog
            return( TRUE );
         }
         if( wParam == 100 )                              //  "Devices" pressed
         {
            GetTAPIDevices( hDlg );                  //  Enumerate TAPI devices
            break;
         }
         if( wParam == 101 ||
             ( LOWORD( wParam ) == 1000 && HIWORD( wParam ) == LBN_DBLCLK ) )
         {
            int iSelection;
            LPSTR lpStr;
                                        //  Get currently selected listbox item
            iSelection = SendDlgItemMessage( hDlg, 1000, LB_GETCURSEL, 0, 0 );
            if( iSelection == LB_ERR )             //  Or exit if there is none
               break;
                                  //  Get memory block associated with the item
            lpStr = (LPSTR)SendDlgItemMessage( hDlg, 1000, LB_GETITEMDATA,
                                               iSelection, 0 );
            if( ! lpStr )                   //  Display error if no device info
               MessageBox( hDlg, "No information on the selected device!",
                           "TapiTest", MB_OK );
            else                                //  Or call the Info dialog box
               DialogBoxParam( hInst, MAKEINTRESOURCE( 10001 ), hDlg,
                               ShowInfoProc, (LPARAM)lpStr );
            break;
         }
         break;
   }
   return( FALSE );
}                                                      //  End of MainDlgProc()

// === Enumerate All TAPI Devices =============================================

BOOL GetTAPIDevices( HWND hWnd )
{
   HLINEAPP hLineApp;
   LONG lStatus;
   int iItem;
   DWORD dwVersion, dwCount, dwDeviceCount;
   LINEEXTENSIONID led;
   LPLINEDEVCAPS lpDc, lpNew;
   LPSTR lpStr;

   
   hLineApp = (HLINEAPP)NULL;
   lStatus = lineInitialize( &hLineApp, hInst,          //  Initialize TAPI DLL
                             (LINECALLBACK)TapiCallbackProc,
                             "TapiTest", &dwDeviceCount );

   if( lStatus < 0 || dwDeviceCount < 1 )
      return( FALSE );                             //  Quit if an error came up

   lpDc = (LPLINEDEVCAPS)GlobalAllocPtr( GHND, 1536 );
   if( ! lpDc )           //  Grab some buffer space or fail if it doesn't work
   {
      lineShutdown( hLineApp );                    //  Reset DLL before leaving
      return( -1 );
   }
                                                 //  Make sure listbox is empty
   DeleteAllDevices( GetDlgItem( hWnd, 1000 ) );
                                         //  Loop through all available devices
   for( dwCount = 0; dwCount < dwDeviceCount; dwCount++ )  
   {
      memset( lpDc, 0, 1536 );                //  Clear the line caps structure
      lpDc->dwTotalSize = 1536;             //  Get a version base to work with
      lStatus = lineNegotiateAPIVersion( hLineApp, dwCount, 
                                         0x00010003, 0x00010004,
                                         &dwVersion, &led );

      if( ! lStatus )                   //  If negotiation worked for this line
      {
         memset( lpDc, 0, 1536 );                       //  Clear out structure
         lpDc->dwTotalSize = 1536;                //  and get line capabilities
         lStatus = lineGetDevCaps( hLineApp, dwCount, dwVersion, 0,
                                   lpDc );
                                                       //  Point to device name
		 lpStr = (LPSTR)( lpDc->dwLineNameOffset + (DWORD)lpDc );
         if( strlen( lpStr ) )                  //  If there is a valid name...
         {                                            //  Add it to the listbox
            iItem = SendDlgItemMessage( hWnd, 1000, LB_ADDSTRING, 0,
                                        (LPARAM)lpStr );
            if( iItem != LB_ERR )
            {                                  //  Alloc some device info space
               lpNew = GlobalAllocPtr( GHND, lpDc->dwUsedSize + 8 );
               if( lpNew )
               {                   //  Copy the info structure to the new block
                  memcpy( lpNew, lpDc, lpDc->dwUsedSize );      //  and save it
                  SendDlgItemMessage( hWnd, 1000, LB_SETITEMDATA, iItem,
                                      (LPARAM)lpNew );
               }
            }
         }
      }
   }
   GlobalFreePtr( lpDc );                //  Free original memory (not listbox)
   lineShutdown( hLineApp );                                 //  Reset TAPI DLL
   return( 0 );
}                                                   //  End of GetTAPIDevices()

// === Remove All Listbox Items and Related Memory Blocks =====================

BOOL DeleteAllDevices( HWND hListbox )
{
   int iCount, i;

   iCount = SendMessage( hListbox, LB_GETCOUNT, 0, 0 );
   if( ! iCount || iCount == LB_ERR )            //  If nothing to delete, exit
      return( TRUE );

   for( i = 0; i < iCount; i++ )                     //  One loop for each item
   {                                          //  Any data with current item #0?
      if( SendMessage( hListbox, LB_GETITEMDATA, 0, 0 ) )
         GlobalFreePtr( (LPVOID)SendMessage( hListbox, LB_GETITEMDATA, 
                        0, 0 ) );               //  If so, free the memory block
      SendMessage( hListbox, LB_DELETESTRING, 0, 0 );         //  Delete item #0
   }
   return( TRUE );
}

// === Show Info Associated With TAPI Device ==================================

BOOL WINAPI ShowInfoProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
   switch( msg )
   {
      case WM_INITDIALOG:
      {
         LINEDEVCAPS *lpDc;
         char szStr[ 256 ];
         LPSTR lpStr;

         lpDc = (LINEDEVCAPS *)lParam;

         lpStr = (LPSTR)( (DWORD)lpDc + lpDc->dwProviderInfoOffset );
         lpStr = lpStr + strlen( lpStr ) + 1;
         if( strlen( lpStr ) )
         {
            strcpy( szStr, "Device: " );
            strcat( szStr, lpStr );
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szStr );
         }

         wsprintf( szStr, "Line ID: %.8x", lpDc->dwPermanentLineID );
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szStr );

         wsprintf( szStr, "Device Addr Count: %d", lpDc->dwNumAddresses );
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szStr );

         wsprintf( szStr, "Max. Data Rate: %d BPS", lpDc->dwMaxRate );
         SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, (LPARAM)szStr );

         if( lpDc->dwMediaModes )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"Supported Media Modes:" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_INTERACTIVEVOICE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Interactive Voice" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_AUTOMATEDVOICE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Automated Voice" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_DATAMODEM )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Data Modem" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_G3FAX )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Group 3 Fax" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_TDD )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Telephony for the Deaf" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_G4FAX )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Group 4 Fax" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_DIGITALDATA )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Digital Data" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_TELETEX )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Teletex Sessions" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_VIDEOTEX )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Videotex Sessions" );

         if( lpDc->dwMediaModes & LINEMEDIAMODE_TELEX )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Telex Sessions" );
         
         if( lpDc->dwBearerModes )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"Supported Bearer Modes:" );

         if( lpDc->dwBearerModes & LINEBEARERMODE_VOICE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   3.1 kHz analog service" );

         if( lpDc->dwBearerModes & LINEBEARERMODE_SPEECH )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   G.711 speech transmission" );

         if( lpDc->dwBearerModes & LINEBEARERMODE_MULTIUSE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   ISDN multiuse service" );

         if( lpDc->dwBearerModes & LINEBEARERMODE_DATA )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Unrestricted data service" );

         if( lpDc->dwBearerModes & LINEBEARERMODE_ALTSPEECHDATA )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Alternate speech or data" );

         if( lpDc->dwBearerModes & LINEBEARERMODE_NONCALLSIGNALING )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Media stream service" );

         if( lpDc->dwBearerModes & LINEBEARERMODE_PASSTHROUGH )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Passthrough service" );

         if( lpDc->dwLineStates )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"Supported Line States:" );

         if( lpDc->dwLineStates & LINEDEVSTATE_RINGING )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line Ringing" );

         if( lpDc->dwLineStates & LINEDEVSTATE_CONNECTED )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line Connected" );

         if( lpDc->dwLineStates & LINEDEVSTATE_DISCONNECTED )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line Disconnected" );

         if( lpDc->dwLineStates & LINEDEVSTATE_MSGWAITON )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   'Msg Waiting' Enabled" );

         if( lpDc->dwLineStates & LINEDEVSTATE_MSGWAITOFF )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   'Msg Waiting' Disabled" );

         if( lpDc->dwLineStates & LINEDEVSTATE_NUMCOMPLETIONS )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Change in Calls Complete" );

         if( lpDc->dwLineStates & LINEDEVSTATE_INSERVICE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   In Service" );

         if( lpDc->dwLineStates & LINEDEVSTATE_OUTOFSERVICE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Out of Service" );

         if( lpDc->dwLineStates & LINEDEVSTATE_MAINTENANCE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line Maintenance" );

         if( lpDc->dwLineStates & LINEDEVSTATE_OPEN )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line Opened" );

         if( lpDc->dwLineStates & LINEDEVSTATE_CLOSE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line Closed" );

         if( lpDc->dwLineStates & LINEDEVSTATE_NUMCALLS )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Change in Call Number" );

         if( lpDc->dwLineStates & LINEDEVSTATE_TERMINALS )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Terminal Settings Change" );

         if( lpDc->dwLineStates & LINEDEVSTATE_ROAMMODE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Roam Mode Change" );

         if( lpDc->dwLineStates & LINEDEVSTATE_BATTERY )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Battery level change" );

         if( lpDc->dwLineStates & LINEDEVSTATE_SIGNAL )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Signal level change" );
         
         if( lpDc->dwLineStates & LINEDEVSTATE_DEVSPECIFIC )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Device Specific Info" );

         if( lpDc->dwLineStates & LINEDEVSTATE_REINIT )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line config changed" );

         if( lpDc->dwLineStates & LINEDEVSTATE_LOCK )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line Lock status change" );

         if( lpDc->dwLineStates & LINEDEVSTATE_CAPSCHANGE )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line DEVCAPS change" );

         if( lpDc->dwLineStates & LINEDEVSTATE_REMOVED )
            SendDlgItemMessage( hDlg, 100, LB_ADDSTRING, 0, 
                                (LPARAM)"   Line Removed" );

         return( TRUE );
      }

      case WM_COMMAND:
         if( wParam == IDOK || wParam == IDCANCEL )
         {
            EndDialog( hDlg, TRUE );
            return( TRUE );
         }
         break;
   }
   return( FALSE );
}

// === TAPI Callback for Initialization Process ===============================

VOID CALLBACK TapiCallbackProc( HANDLE hDevice, DWORD dwMsg, DWORD dwInstance,
                               DWORD dwParam1, DWORD dwParam2, DWORD dwParam3 )
{
   return;
}

//  End of TTEST2.C
